from __future__ import annotations
from win32more import ARCH, Annotated, Boolean, Byte, Bytes, Char, ComPtr, ConstantLazyLoader, Double, Enum, FAILED, Guid, Int16, Int32, Int64, IntPtr, POINTER, SByte, SUCCEEDED, Single, String, Structure, UInt16, UInt32, UInt64, UIntPtr, UnicodeAlias, Union, Void, VoidPtr, cfunctype, cfunctype_pointer, commethod, make_ready, winfunctype, winfunctype_pointer
import win32more.Windows.Win32.Foundation
import win32more.Windows.Win32.Networking.WebSocket
WEB_SOCKET_MAX_CLOSE_REASON_LENGTH: UInt32 = 123
@winfunctype('websocket.dll')
def WebSocketCreateClientHandle(pProperties: POINTER(win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_PROPERTY), ulPropertyCount: UInt32, phWebSocket: POINTER(win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_HANDLE)) -> win32more.Windows.Win32.Foundation.HRESULT: ...
@winfunctype('websocket.dll')
def WebSocketBeginClientHandshake(hWebSocket: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_HANDLE, pszSubprotocols: POINTER(win32more.Windows.Win32.Foundation.PSTR), ulSubprotocolCount: UInt32, pszExtensions: POINTER(win32more.Windows.Win32.Foundation.PSTR), ulExtensionCount: UInt32, pInitialHeaders: POINTER(win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_HTTP_HEADER), ulInitialHeaderCount: UInt32, pAdditionalHeaders: POINTER(POINTER(win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_HTTP_HEADER)), pulAdditionalHeaderCount: POINTER(UInt32)) -> win32more.Windows.Win32.Foundation.HRESULT: ...
@winfunctype('websocket.dll')
def WebSocketEndClientHandshake(hWebSocket: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_HANDLE, pResponseHeaders: POINTER(win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_HTTP_HEADER), ulReponseHeaderCount: UInt32, pulSelectedExtensions: POINTER(UInt32), pulSelectedExtensionCount: POINTER(UInt32), pulSelectedSubprotocol: POINTER(UInt32)) -> win32more.Windows.Win32.Foundation.HRESULT: ...
@winfunctype('websocket.dll')
def WebSocketCreateServerHandle(pProperties: POINTER(win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_PROPERTY), ulPropertyCount: UInt32, phWebSocket: POINTER(win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_HANDLE)) -> win32more.Windows.Win32.Foundation.HRESULT: ...
@winfunctype('websocket.dll')
def WebSocketBeginServerHandshake(hWebSocket: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_HANDLE, pszSubprotocolSelected: win32more.Windows.Win32.Foundation.PSTR, pszExtensionSelected: POINTER(win32more.Windows.Win32.Foundation.PSTR), ulExtensionSelectedCount: UInt32, pRequestHeaders: POINTER(win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_HTTP_HEADER), ulRequestHeaderCount: UInt32, pResponseHeaders: POINTER(POINTER(win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_HTTP_HEADER)), pulResponseHeaderCount: POINTER(UInt32)) -> win32more.Windows.Win32.Foundation.HRESULT: ...
@winfunctype('websocket.dll')
def WebSocketEndServerHandshake(hWebSocket: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_HANDLE) -> win32more.Windows.Win32.Foundation.HRESULT: ...
@winfunctype('websocket.dll')
def WebSocketSend(hWebSocket: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_HANDLE, BufferType: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_BUFFER_TYPE, pBuffer: POINTER(win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_BUFFER), Context: VoidPtr) -> win32more.Windows.Win32.Foundation.HRESULT: ...
@winfunctype('websocket.dll')
def WebSocketReceive(hWebSocket: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_HANDLE, pBuffer: POINTER(win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_BUFFER), pvContext: VoidPtr) -> win32more.Windows.Win32.Foundation.HRESULT: ...
@winfunctype('websocket.dll')
def WebSocketGetAction(hWebSocket: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_HANDLE, eActionQueue: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_ACTION_QUEUE, pDataBuffers: POINTER(win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_BUFFER), pulDataBufferCount: POINTER(UInt32), pAction: POINTER(win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_ACTION), pBufferType: POINTER(win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_BUFFER_TYPE), pvApplicationContext: POINTER(VoidPtr), pvActionContext: POINTER(VoidPtr)) -> win32more.Windows.Win32.Foundation.HRESULT: ...
@winfunctype('websocket.dll')
def WebSocketCompleteAction(hWebSocket: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_HANDLE, pvActionContext: VoidPtr, ulBytesTransferred: UInt32) -> Void: ...
@winfunctype('websocket.dll')
def WebSocketAbortHandle(hWebSocket: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_HANDLE) -> Void: ...
@winfunctype('websocket.dll')
def WebSocketDeleteHandle(hWebSocket: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_HANDLE) -> Void: ...
@winfunctype('websocket.dll')
def WebSocketGetGlobalProperty(eType: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_PROPERTY_TYPE, pvValue: VoidPtr, ulSize: POINTER(UInt32)) -> win32more.Windows.Win32.Foundation.HRESULT: ...
WEB_SOCKET_ACTION = Int32
WEB_SOCKET_NO_ACTION: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_ACTION = 0
WEB_SOCKET_SEND_TO_NETWORK_ACTION: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_ACTION = 1
WEB_SOCKET_INDICATE_SEND_COMPLETE_ACTION: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_ACTION = 2
WEB_SOCKET_RECEIVE_FROM_NETWORK_ACTION: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_ACTION = 3
WEB_SOCKET_INDICATE_RECEIVE_COMPLETE_ACTION: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_ACTION = 4
WEB_SOCKET_ACTION_QUEUE = Int32
WEB_SOCKET_SEND_ACTION_QUEUE: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_ACTION_QUEUE = 1
WEB_SOCKET_RECEIVE_ACTION_QUEUE: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_ACTION_QUEUE = 2
WEB_SOCKET_ALL_ACTION_QUEUE: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_ACTION_QUEUE = 3
class WEB_SOCKET_BUFFER(Union):
    Data: _Data_e__Struct
    CloseStatus: _CloseStatus_e__Struct
    class _Data_e__Struct(Structure):
        pbBuffer: POINTER(Byte)
        ulBufferLength: UInt32
    class _CloseStatus_e__Struct(Structure):
        pbReason: POINTER(Byte)
        ulReasonLength: UInt32
        usStatus: UInt16
WEB_SOCKET_BUFFER_TYPE = Int32
WEB_SOCKET_UTF8_MESSAGE_BUFFER_TYPE: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_BUFFER_TYPE = -2147483648
WEB_SOCKET_UTF8_FRAGMENT_BUFFER_TYPE: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_BUFFER_TYPE = -2147483647
WEB_SOCKET_BINARY_MESSAGE_BUFFER_TYPE: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_BUFFER_TYPE = -2147483646
WEB_SOCKET_BINARY_FRAGMENT_BUFFER_TYPE: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_BUFFER_TYPE = -2147483645
WEB_SOCKET_CLOSE_BUFFER_TYPE: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_BUFFER_TYPE = -2147483644
WEB_SOCKET_PING_PONG_BUFFER_TYPE: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_BUFFER_TYPE = -2147483643
WEB_SOCKET_UNSOLICITED_PONG_BUFFER_TYPE: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_BUFFER_TYPE = -2147483642
WEB_SOCKET_CLOSE_STATUS = Int32
WEB_SOCKET_SUCCESS_CLOSE_STATUS: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_CLOSE_STATUS = 1000
WEB_SOCKET_ENDPOINT_UNAVAILABLE_CLOSE_STATUS: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_CLOSE_STATUS = 1001
WEB_SOCKET_PROTOCOL_ERROR_CLOSE_STATUS: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_CLOSE_STATUS = 1002
WEB_SOCKET_INVALID_DATA_TYPE_CLOSE_STATUS: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_CLOSE_STATUS = 1003
WEB_SOCKET_EMPTY_CLOSE_STATUS: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_CLOSE_STATUS = 1005
WEB_SOCKET_ABORTED_CLOSE_STATUS: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_CLOSE_STATUS = 1006
WEB_SOCKET_INVALID_PAYLOAD_CLOSE_STATUS: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_CLOSE_STATUS = 1007
WEB_SOCKET_POLICY_VIOLATION_CLOSE_STATUS: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_CLOSE_STATUS = 1008
WEB_SOCKET_MESSAGE_TOO_BIG_CLOSE_STATUS: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_CLOSE_STATUS = 1009
WEB_SOCKET_UNSUPPORTED_EXTENSIONS_CLOSE_STATUS: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_CLOSE_STATUS = 1010
WEB_SOCKET_SERVER_ERROR_CLOSE_STATUS: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_CLOSE_STATUS = 1011
WEB_SOCKET_SECURE_HANDSHAKE_ERROR_CLOSE_STATUS: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_CLOSE_STATUS = 1015
WEB_SOCKET_HANDLE = VoidPtr
class WEB_SOCKET_HTTP_HEADER(Structure):
    pcName: win32more.Windows.Win32.Foundation.PSTR
    ulNameLength: UInt32
    pcValue: win32more.Windows.Win32.Foundation.PSTR
    ulValueLength: UInt32
class WEB_SOCKET_PROPERTY(Structure):
    Type: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_PROPERTY_TYPE
    pvValue: VoidPtr
    ulValueSize: UInt32
WEB_SOCKET_PROPERTY_TYPE = Int32
WEB_SOCKET_RECEIVE_BUFFER_SIZE_PROPERTY_TYPE: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_PROPERTY_TYPE = 0
WEB_SOCKET_SEND_BUFFER_SIZE_PROPERTY_TYPE: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_PROPERTY_TYPE = 1
WEB_SOCKET_DISABLE_MASKING_PROPERTY_TYPE: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_PROPERTY_TYPE = 2
WEB_SOCKET_ALLOCATED_BUFFER_PROPERTY_TYPE: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_PROPERTY_TYPE = 3
WEB_SOCKET_DISABLE_UTF8_VERIFICATION_PROPERTY_TYPE: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_PROPERTY_TYPE = 4
WEB_SOCKET_KEEPALIVE_INTERVAL_PROPERTY_TYPE: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_PROPERTY_TYPE = 5
WEB_SOCKET_SUPPORTED_VERSIONS_PROPERTY_TYPE: win32more.Windows.Win32.Networking.WebSocket.WEB_SOCKET_PROPERTY_TYPE = 6


make_ready(__name__)



from __future__ import annotations

import os
import sys
import webbrowser
from pathlib import Path

import win32more.mddbootstrap
import win32more.Windows.Win32.Foundation
import win32more.Windows.Win32.Storage.Packaging.Appx
from win32more import ARCH, FAILED, Char, Int32, String, UInt32, Void, WinError, make_ready, winfunctype
from win32more.Windows.Win32.Foundation import (
    APPMODEL_ERROR_NO_PACKAGE,
    ERROR_INSUFFICIENT_BUFFER,
    ERROR_SUCCESS,
    S_OK,
    STATEREPOSITORY_E_DEPENDENCY_NOT_RESOLVED,
)
from win32more.Windows.Win32.Storage.Packaging.Appx import (
    PACKAGE_VERSION,
    PACKAGEDEPENDENCY_CONTEXT,
    AddPackageDependency,
    AddPackageDependencyOptions_None,
    CreatePackageDependencyOptions_None,
    GetCurrentPackageFullName,
    GetPackagePathByFullName,
    GetPackagesByPackageFamily,
    PackageDependencyLifetimeKind_Process,
    PackageDependencyProcessorArchitectures_Arm64,
    PackageDependencyProcessorArchitectures_X64,
    PackageDependencyProcessorArchitectures_X86,
    TryCreatePackageDependency,
)
from win32more.Windows.Win32.UI.WindowsAndMessaging import IDYES, MB_ICONERROR, MB_YESNO, MessageBox

_module = sys.modules[__name__]

# TODO: keep sync with WindowsAppSDK-VersionInfo.h
# VERSION: 1.6.241114003
WINDOWSAPPSDK_RELEASE_MAJORMINOR = 0x00010006
WINDOWSAPPSDK_RELEASE_VERSION_SHORTTAG_W = ""
WINDOWSAPPSDK_RUNTIME_VERSION_UINT64 = 0x1770013E09000000

if ARCH == "ARM64":
    PackageDependencyProcessorArchitectures_Current = PackageDependencyProcessorArchitectures_Arm64
    os.add_dll_directory(os.path.dirname(__file__) + "\\dll\\arm64")
elif ARCH == "X64":
    PackageDependencyProcessorArchitectures_Current = PackageDependencyProcessorArchitectures_X64
    os.add_dll_directory(os.path.dirname(__file__) + "\\dll\\x64")
elif ARCH == "X86":
    PackageDependencyProcessorArchitectures_Current = PackageDependencyProcessorArchitectures_X86
    os.add_dll_directory(os.path.dirname(__file__) + "\\dll\\x86")
else:
    assert False

MddBootstrapInitializeOptions = Int32
MddBootstrapInitializeOptions_None: win32more.mddbootstrap.MddBootstrapInitializeOptions = 0
MddBootstrapInitializeOptions_OnError_DebugBreak: win32more.mddbootstrap.MddBootstrapInitializeOptions = 1
MddBootstrapInitializeOptions_OnError_DebugBreak_IfDebuggerAttached: win32more.mddbootstrap.MddBootstrapInitializeOptions = 2
MddBootstrapInitializeOptions_OnError_FailFast: win32more.mddbootstrap.MddBootstrapInitializeOptions = 4
MddBootstrapInitializeOptions_OnNoMatch_ShowUI: win32more.mddbootstrap.MddBootstrapInitializeOptions = 8
MddBootstrapInitializeOptions_OnPackageIdentity_NOOP: win32more.mddbootstrap.MddBootstrapInitializeOptions = 16


@winfunctype("Microsoft.WindowsAppRuntime.Bootstrap.dll", entry_point="MddBootstrapInitialize")
def _MddBootstrapInitialize(
    majorMinorVersion: UInt32,
    versionTag: String,
    minVersion: win32more.Windows.Win32.Storage.Packaging.Appx.PACKAGE_VERSION,
) -> win32more.Windows.Win32.Foundation.HRESULT: ...


@winfunctype("Microsoft.WindowsAppRuntime.Bootstrap.dll", entry_point="MddBootstrapInitialize2")
def _MddBootstrapInitialize2(
    majorMinorVersion: UInt32,
    versionTag: String,
    minVersion: win32more.Windows.Win32.Storage.Packaging.Appx.PACKAGE_VERSION,
    options: win32more.mddbootstrap.MddBootstrapInitializeOptions,
) -> win32more.Windows.Win32.Foundation.HRESULT: ...


@winfunctype("Microsoft.WindowsAppRuntime.Bootstrap.dll", entry_point="MddBootstrapShutdown")
def _MddBootstrapShutdown() -> Void: ...


def MddBootstrapInitialize(major_minor_version: int, version_tag: str, min_version: PACKAGE_VERSION) -> int:
    return MddBootstrapInitialize2(major_minor_version, version_tag, min_version, MddBootstrapInitializeOptions_None)


def MddBootstrapInitialize2(
    major_minor_version: int, version_tag: str, min_version: PACKAGE_VERSION, options: int
) -> int:
    if _IsPackagedProcess() and not _IsWin11():
        raise RuntimeError("Packaged process is not supported before Windows 11")
    elif _IsWin11():
        hr = _Initialize_Win11(_GetFrameworkPackageFamilyName(major_minor_version, version_tag), min_version)
        if FAILED(hr):
            if hr == STATEREPOSITORY_E_DEPENDENCY_NOT_RESOLVED:
                if options & MddBootstrapInitializeOptions_OnNoMatch_ShowUI:
                    _OnNoMatch_ShowUI(major_minor_version, version_tag, min_version)
            return hr
        return S_OK
    else:
        return _module._MddBootstrapInitialize2(major_minor_version, version_tag, min_version, options)


def MddBootstrapShutdown() -> None:
    if _IsPackagedProcess() and not _IsWin11():
        raise RuntimeError("Packaged process is not supported before Windows 11")
    elif _IsWin11():
        pass
    else:
        return _module._MddBootstrapShutdown()


def _IsWin11() -> bool:
    # FIXME: RoActivateInstance() failes
    return False
    # return sys.getwindowsversion() >= (10, 0, 22000)


def _IsPackagedProcess() -> bool:
    return GetCurrentPackageFullName(UInt32(), None) != APPMODEL_ERROR_NO_PACKAGE


def _Initialize_Win11(family_name: str, min_version: PACKAGE_VERSION) -> int:
    package_dependency_id = String()
    hr = TryCreatePackageDependency(
        None,
        family_name,
        min_version,
        PackageDependencyProcessorArchitectures_Current,
        PackageDependencyLifetimeKind_Process,
        None,
        CreatePackageDependencyOptions_None,
        package_dependency_id,
    )
    if FAILED(hr):
        return hr

    MDD_PACKAGE_DEPENDENCY_RANK_DEFAULT = 0
    package_dependency_context = PACKAGEDEPENDENCY_CONTEXT()
    hr = AddPackageDependency(
        package_dependency_id,
        MDD_PACKAGE_DEPENDENCY_RANK_DEFAULT,
        AddPackageDependencyOptions_None,
        package_dependency_context,
        None,
    )
    if FAILED(hr):
        return hr

    return S_OK


# https://learn.microsoft.com/en-us/windows/apps/desktop/modernize/package-identity-overview
def _GetFrameworkPackageFamilyName(major_minor_version: int, version_tag: PACKAGE_VERSION) -> str:
    major_version = major_minor_version >> 16
    minor_version = major_minor_version & 0xFFFF
    version_tag_delimiter = "" if version_tag == "" else "-"
    name = f"Microsoft.WindowsAppRuntime.{major_version}.{minor_version}{version_tag_delimiter}{version_tag}"
    publisher_id = "8wekyb3d8bbwe"
    return f"{name}_{publisher_id}"


def _GetFrameworkPackageDirectory(
    major_minor_version: int, version_tag: str, min_version: PACKAGE_VERSION
) -> str | None:
    family_name = _GetFrameworkPackageFamilyName(major_minor_version, version_tag)
    full_name = _ResolvePackageDependency(family_name, min_version, ARCH)
    if full_name is None:
        return None
    return _GetPackagePathByFullName(full_name)


def _ResolvePackageDependency(family_name: str, min_version: PACKAGE_VERSION, arch: str) -> str | None:
    bestfit = None
    latest_version = None

    for full_name in _GetPackagesByPackageFamily(family_name):
        name, version, architecture, resource_id, publisher_id = full_name.split("_")
        major, minor, build, revision = [int(x) for x in version.split(".")]
        if architecture.upper() != arch:
            continue
        if (major, minor, build, revision) < (
            min_version.Major,
            min_version.Minor,
            min_version.Build,
            min_version.Revision,
        ):
            continue
        if latest_version is None or latest_version < (major, minor, build, revision):
            bestfit = full_name
            latest_version = (major, minor, build, revision)

    return bestfit


def _OnNoMatch_ShowUI(major_minor_version: int, version_tag: str, min_version: PACKAGE_VERSION) -> None:
    caption = f"{Path(sys.executable).name} - This application could not be started"
    text = f"""This application requires the Windows App Runtime
Version {major_minor_version >> 16}.{major_minor_version & 0xFFFF}
(MSIX package version >= {min_version.Major}.{min_version.Minor}.{min_version.Build}.{min_version.Revision})

Do you want to install a compatible Windows App Runtime now?"""
    r = MessageBox(0, text, caption, MB_YESNO | MB_ICONERROR)
    if r == IDYES:
        webbrowser.open("https://docs.microsoft.com/windows/apps/windows-app-sdk/downloads")


def _GetPackagesByPackageFamily(family_name: str) -> list[str]:
    count = UInt32()
    buffer_length = UInt32()
    r = GetPackagesByPackageFamily(family_name, count, None, buffer_length, None)
    if r == ERROR_SUCCESS:
        return []  # not found
    elif r != ERROR_INSUFFICIENT_BUFFER:
        raise WinError(r)

    full_names = (String * count.value)()
    buffer = (Char * buffer_length.value)()
    r = GetPackagesByPackageFamily(family_name, count, full_names, buffer_length, buffer)
    if r != ERROR_SUCCESS:
        raise WinError(r)

    return full_names[: count.value]


def _GetPackagePathByFullName(full_name: str) -> str:
    path_length = UInt32()
    r = GetPackagePathByFullName(full_name, path_length, None)
    if r != ERROR_INSUFFICIENT_BUFFER:
        raise WinError(r)

    path = (Char * path_length.value)()
    r = GetPackagePathByFullName(full_name, path_length, path)
    if r != ERROR_SUCCESS:
        raise WinError(r)

    return path.value


make_ready(__name__)



from __future__ import annotations
from win32more import ARCH, Annotated, Boolean, Byte, Bytes, Char, ComPtr, ConstantLazyLoader, Double, Enum, FAILED, Guid, Int16, Int32, Int64, IntPtr, POINTER, SByte, SUCCEEDED, Single, String, Structure, UInt16, UInt32, UInt64, UIntPtr, UnicodeAlias, Union, Void, VoidPtr, cfunctype, cfunctype_pointer, commethod, make_ready, winfunctype, winfunctype_pointer
import win32more.Windows.Win32.Foundation
import win32more.Windows.Win32.Security
import win32more.Windows.Win32.System.JobObjects
import win32more.Windows.Win32.System.Threading
@winfunctype('KERNEL32.dll')
def IsProcessInJob(ProcessHandle: win32more.Windows.Win32.Foundation.HANDLE, JobHandle: win32more.Windows.Win32.Foundation.HANDLE, Result: POINTER(win32more.Windows.Win32.Foundation.BOOL)) -> win32more.Windows.Win32.Foundation.BOOL: ...
@winfunctype('KERNEL32.dll')
def CreateJobObjectW(lpJobAttributes: POINTER(win32more.Windows.Win32.Security.SECURITY_ATTRIBUTES), lpName: win32more.Windows.Win32.Foundation.PWSTR) -> win32more.Windows.Win32.Foundation.HANDLE: ...
CreateJobObject = UnicodeAlias('CreateJobObjectW')
@winfunctype('KERNEL32.dll')
def FreeMemoryJobObject(Buffer: VoidPtr) -> Void: ...
@winfunctype('KERNEL32.dll')
def OpenJobObjectW(dwDesiredAccess: UInt32, bInheritHandle: win32more.Windows.Win32.Foundation.BOOL, lpName: win32more.Windows.Win32.Foundation.PWSTR) -> win32more.Windows.Win32.Foundation.HANDLE: ...
OpenJobObject = UnicodeAlias('OpenJobObjectW')
@winfunctype('KERNEL32.dll')
def AssignProcessToJobObject(hJob: win32more.Windows.Win32.Foundation.HANDLE, hProcess: win32more.Windows.Win32.Foundation.HANDLE) -> win32more.Windows.Win32.Foundation.BOOL: ...
@winfunctype('KERNEL32.dll')
def TerminateJobObject(hJob: win32more.Windows.Win32.Foundation.HANDLE, uExitCode: UInt32) -> win32more.Windows.Win32.Foundation.BOOL: ...
@winfunctype('KERNEL32.dll')
def SetInformationJobObject(hJob: win32more.Windows.Win32.Foundation.HANDLE, JobObjectInformationClass: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS, lpJobObjectInformation: VoidPtr, cbJobObjectInformationLength: UInt32) -> win32more.Windows.Win32.Foundation.BOOL: ...
@winfunctype('KERNEL32.dll')
def SetIoRateControlInformationJobObject(hJob: win32more.Windows.Win32.Foundation.HANDLE, IoRateControlInfo: POINTER(win32more.Windows.Win32.System.JobObjects.JOBOBJECT_IO_RATE_CONTROL_INFORMATION)) -> UInt32: ...
@winfunctype('KERNEL32.dll')
def QueryInformationJobObject(hJob: win32more.Windows.Win32.Foundation.HANDLE, JobObjectInformationClass: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS, lpJobObjectInformation: VoidPtr, cbJobObjectInformationLength: UInt32, lpReturnLength: POINTER(UInt32)) -> win32more.Windows.Win32.Foundation.BOOL: ...
@winfunctype('KERNEL32.dll')
def QueryIoRateControlInformationJobObject(hJob: win32more.Windows.Win32.Foundation.HANDLE, VolumeName: win32more.Windows.Win32.Foundation.PWSTR, InfoBlocks: POINTER(POINTER(win32more.Windows.Win32.System.JobObjects.JOBOBJECT_IO_RATE_CONTROL_INFORMATION)), InfoBlockCount: POINTER(UInt32)) -> UInt32: ...
@winfunctype('USER32.dll')
def UserHandleGrantAccess(hUserHandle: win32more.Windows.Win32.Foundation.HANDLE, hJob: win32more.Windows.Win32.Foundation.HANDLE, bGrant: win32more.Windows.Win32.Foundation.BOOL) -> win32more.Windows.Win32.Foundation.BOOL: ...
@winfunctype('KERNEL32.dll')
def CreateJobObjectA(lpJobAttributes: POINTER(win32more.Windows.Win32.Security.SECURITY_ATTRIBUTES), lpName: win32more.Windows.Win32.Foundation.PSTR) -> win32more.Windows.Win32.Foundation.HANDLE: ...
@winfunctype('KERNEL32.dll')
def OpenJobObjectA(dwDesiredAccess: UInt32, bInheritHandle: win32more.Windows.Win32.Foundation.BOOL, lpName: win32more.Windows.Win32.Foundation.PSTR) -> win32more.Windows.Win32.Foundation.HANDLE: ...
@winfunctype('KERNEL32.dll')
def CreateJobSet(NumJob: UInt32, UserJobSet: POINTER(win32more.Windows.Win32.System.JobObjects.JOB_SET_ARRAY), Flags: UInt32) -> win32more.Windows.Win32.Foundation.BOOL: ...
JOBOBJECTINFOCLASS = Int32
JobObjectBasicAccountingInformation: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 1
JobObjectBasicLimitInformation: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 2
JobObjectBasicProcessIdList: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 3
JobObjectBasicUIRestrictions: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 4
JobObjectSecurityLimitInformation: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 5
JobObjectEndOfJobTimeInformation: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 6
JobObjectAssociateCompletionPortInformation: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 7
JobObjectBasicAndIoAccountingInformation: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 8
JobObjectExtendedLimitInformation: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 9
JobObjectJobSetInformation: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 10
JobObjectGroupInformation: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 11
JobObjectNotificationLimitInformation: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 12
JobObjectLimitViolationInformation: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 13
JobObjectGroupInformationEx: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 14
JobObjectCpuRateControlInformation: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 15
JobObjectCompletionFilter: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 16
JobObjectCompletionCounter: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 17
JobObjectReserved1Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 18
JobObjectReserved2Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 19
JobObjectReserved3Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 20
JobObjectReserved4Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 21
JobObjectReserved5Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 22
JobObjectReserved6Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 23
JobObjectReserved7Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 24
JobObjectReserved8Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 25
JobObjectReserved9Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 26
JobObjectReserved10Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 27
JobObjectReserved11Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 28
JobObjectReserved12Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 29
JobObjectReserved13Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 30
JobObjectReserved14Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 31
JobObjectNetRateControlInformation: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 32
JobObjectNotificationLimitInformation2: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 33
JobObjectLimitViolationInformation2: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 34
JobObjectCreateSilo: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 35
JobObjectSiloBasicInformation: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 36
JobObjectReserved15Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 37
JobObjectReserved16Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 38
JobObjectReserved17Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 39
JobObjectReserved18Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 40
JobObjectReserved19Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 41
JobObjectReserved20Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 42
JobObjectReserved21Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 43
JobObjectReserved22Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 44
JobObjectReserved23Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 45
JobObjectReserved24Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 46
JobObjectReserved25Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 47
JobObjectReserved26Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 48
JobObjectReserved27Information: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 49
MaxJobObjectInfoClass: win32more.Windows.Win32.System.JobObjects.JOBOBJECTINFOCLASS = 50
class JOBOBJECT_ASSOCIATE_COMPLETION_PORT(Structure):
    CompletionKey: VoidPtr
    CompletionPort: win32more.Windows.Win32.Foundation.HANDLE
class JOBOBJECT_BASIC_ACCOUNTING_INFORMATION(Structure):
    TotalUserTime: Int64
    TotalKernelTime: Int64
    ThisPeriodTotalUserTime: Int64
    ThisPeriodTotalKernelTime: Int64
    TotalPageFaultCount: UInt32
    TotalProcesses: UInt32
    ActiveProcesses: UInt32
    TotalTerminatedProcesses: UInt32
class JOBOBJECT_BASIC_AND_IO_ACCOUNTING_INFORMATION(Structure):
    BasicInfo: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_BASIC_ACCOUNTING_INFORMATION
    IoInfo: win32more.Windows.Win32.System.Threading.IO_COUNTERS
class JOBOBJECT_BASIC_LIMIT_INFORMATION(Structure):
    PerProcessUserTimeLimit: Int64
    PerJobUserTimeLimit: Int64
    LimitFlags: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT
    MinimumWorkingSetSize: UIntPtr
    MaximumWorkingSetSize: UIntPtr
    ActiveProcessLimit: UInt32
    Affinity: UIntPtr
    PriorityClass: UInt32
    SchedulingClass: UInt32
class JOBOBJECT_BASIC_PROCESS_ID_LIST(Structure):
    NumberOfAssignedProcesses: UInt32
    NumberOfProcessIdsInList: UInt32
    ProcessIdList: UIntPtr * 1
class JOBOBJECT_BASIC_UI_RESTRICTIONS(Structure):
    UIRestrictionsClass: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_UILIMIT
class JOBOBJECT_CPU_RATE_CONTROL_INFORMATION(Structure):
    ControlFlags: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_CPU_RATE_CONTROL
    Anonymous: _Anonymous_e__Union
    class _Anonymous_e__Union(Union):
        CpuRate: UInt32
        Weight: UInt32
        Anonymous: _Anonymous_e__Struct
        class _Anonymous_e__Struct(Structure):
            MinRate: UInt16
            MaxRate: UInt16
class JOBOBJECT_END_OF_JOB_TIME_INFORMATION(Structure):
    EndOfJobTimeAction: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_TERMINATE_AT_END_ACTION
class JOBOBJECT_EXTENDED_LIMIT_INFORMATION(Structure):
    BasicLimitInformation: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_BASIC_LIMIT_INFORMATION
    IoInfo: win32more.Windows.Win32.System.Threading.IO_COUNTERS
    ProcessMemoryLimit: UIntPtr
    JobMemoryLimit: UIntPtr
    PeakProcessMemoryUsed: UIntPtr
    PeakJobMemoryUsed: UIntPtr
JOBOBJECT_IO_ATTRIBUTION_CONTROL_FLAGS = Int32
JOBOBJECT_IO_ATTRIBUTION_CONTROL_ENABLE: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_IO_ATTRIBUTION_CONTROL_FLAGS = 1
JOBOBJECT_IO_ATTRIBUTION_CONTROL_DISABLE: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_IO_ATTRIBUTION_CONTROL_FLAGS = 2
JOBOBJECT_IO_ATTRIBUTION_CONTROL_VALID_FLAGS: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_IO_ATTRIBUTION_CONTROL_FLAGS = 3
class JOBOBJECT_IO_ATTRIBUTION_INFORMATION(Structure):
    ControlFlags: UInt32
    ReadStats: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_IO_ATTRIBUTION_STATS
    WriteStats: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_IO_ATTRIBUTION_STATS
class JOBOBJECT_IO_ATTRIBUTION_STATS(Structure):
    IoCount: UIntPtr
    TotalNonOverlappedQueueTime: UInt64
    TotalNonOverlappedServiceTime: UInt64
    TotalSize: UInt64
class JOBOBJECT_IO_RATE_CONTROL_INFORMATION(Structure):
    MaxIops: Int64
    MaxBandwidth: Int64
    ReservationIops: Int64
    VolumeName: win32more.Windows.Win32.Foundation.PWSTR
    BaseIoSize: UInt32
    ControlFlags: UInt32
class JOBOBJECT_IO_RATE_CONTROL_INFORMATION_NATIVE_V1(Structure):
    MaxIops: Int64
    MaxBandwidth: Int64
    ReservationIops: Int64
    VolumeName: win32more.Windows.Win32.Foundation.PWSTR
    BaseIoSize: UInt32
    ControlFlags: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_IO_RATE_CONTROL_FLAGS
    VolumeNameLength: UInt16
class JOBOBJECT_IO_RATE_CONTROL_INFORMATION_NATIVE_V2(Structure):
    MaxIops: Int64
    MaxBandwidth: Int64
    ReservationIops: Int64
    VolumeName: win32more.Windows.Win32.Foundation.PWSTR
    BaseIoSize: UInt32
    ControlFlags: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_IO_RATE_CONTROL_FLAGS
    VolumeNameLength: UInt16
    CriticalReservationIops: Int64
    ReservationBandwidth: Int64
    CriticalReservationBandwidth: Int64
    MaxTimePercent: Int64
    ReservationTimePercent: Int64
    CriticalReservationTimePercent: Int64
class JOBOBJECT_IO_RATE_CONTROL_INFORMATION_NATIVE_V3(Structure):
    MaxIops: Int64
    MaxBandwidth: Int64
    ReservationIops: Int64
    VolumeName: win32more.Windows.Win32.Foundation.PWSTR
    BaseIoSize: UInt32
    ControlFlags: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_IO_RATE_CONTROL_FLAGS
    VolumeNameLength: UInt16
    CriticalReservationIops: Int64
    ReservationBandwidth: Int64
    CriticalReservationBandwidth: Int64
    MaxTimePercent: Int64
    ReservationTimePercent: Int64
    CriticalReservationTimePercent: Int64
    SoftMaxIops: Int64
    SoftMaxBandwidth: Int64
    SoftMaxTimePercent: Int64
    LimitExcessNotifyIops: Int64
    LimitExcessNotifyBandwidth: Int64
    LimitExcessNotifyTimePercent: Int64
class JOBOBJECT_JOBSET_INFORMATION(Structure):
    MemberLevel: UInt32
class JOBOBJECT_LIMIT_VIOLATION_INFORMATION(Structure):
    LimitFlags: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT
    ViolationLimitFlags: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT
    IoReadBytes: UInt64
    IoReadBytesLimit: UInt64
    IoWriteBytes: UInt64
    IoWriteBytesLimit: UInt64
    PerJobUserTime: Int64
    PerJobUserTimeLimit: Int64
    JobMemory: UInt64
    JobMemoryLimit: UInt64
    RateControlTolerance: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE
    RateControlToleranceLimit: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE
class JOBOBJECT_LIMIT_VIOLATION_INFORMATION_2(Structure):
    LimitFlags: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT
    ViolationLimitFlags: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT
    IoReadBytes: UInt64
    IoReadBytesLimit: UInt64
    IoWriteBytes: UInt64
    IoWriteBytesLimit: UInt64
    PerJobUserTime: Int64
    PerJobUserTimeLimit: Int64
    JobMemory: UInt64
    Anonymous1: _Anonymous1_e__Union
    Anonymous2: _Anonymous2_e__Union
    Anonymous3: _Anonymous3_e__Union
    JobLowMemoryLimit: UInt64
    IoRateControlTolerance: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE
    IoRateControlToleranceLimit: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE
    NetRateControlTolerance: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE
    NetRateControlToleranceLimit: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE
    class _Anonymous1_e__Union(Union):
        JobHighMemoryLimit: UInt64
        JobMemoryLimit: UInt64
    class _Anonymous2_e__Union(Union):
        RateControlTolerance: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE
        CpuRateControlTolerance: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE
    class _Anonymous3_e__Union(Union):
        RateControlToleranceLimit: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE
        CpuRateControlToleranceLimit: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE
class JOBOBJECT_NET_RATE_CONTROL_INFORMATION(Structure):
    MaxBandwidth: UInt64
    ControlFlags: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_NET_RATE_CONTROL_FLAGS
    DscpTag: Byte
class JOBOBJECT_NOTIFICATION_LIMIT_INFORMATION(Structure):
    IoReadBytesLimit: UInt64
    IoWriteBytesLimit: UInt64
    PerJobUserTimeLimit: Int64
    JobMemoryLimit: UInt64
    RateControlTolerance: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE
    RateControlToleranceInterval: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE_INTERVAL
    LimitFlags: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT
class JOBOBJECT_NOTIFICATION_LIMIT_INFORMATION_2(Structure):
    IoReadBytesLimit: UInt64
    IoWriteBytesLimit: UInt64
    PerJobUserTimeLimit: Int64
    Anonymous1: _Anonymous1_e__Union
    Anonymous2: _Anonymous2_e__Union
    Anonymous3: _Anonymous3_e__Union
    LimitFlags: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT
    IoRateControlTolerance: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE
    JobLowMemoryLimit: UInt64
    IoRateControlToleranceInterval: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE_INTERVAL
    NetRateControlTolerance: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE
    NetRateControlToleranceInterval: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE_INTERVAL
    class _Anonymous1_e__Union(Union):
        JobHighMemoryLimit: UInt64
        JobMemoryLimit: UInt64
    class _Anonymous2_e__Union(Union):
        RateControlTolerance: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE
        CpuRateControlTolerance: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE
    class _Anonymous3_e__Union(Union):
        RateControlToleranceInterval: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE_INTERVAL
        CpuRateControlToleranceInterval: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE_INTERVAL
JOBOBJECT_RATE_CONTROL_TOLERANCE = Int32
ToleranceLow: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE = 1
ToleranceMedium: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE = 2
ToleranceHigh: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE = 3
JOBOBJECT_RATE_CONTROL_TOLERANCE_INTERVAL = Int32
ToleranceIntervalShort: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE_INTERVAL = 1
ToleranceIntervalMedium: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE_INTERVAL = 2
ToleranceIntervalLong: win32more.Windows.Win32.System.JobObjects.JOBOBJECT_RATE_CONTROL_TOLERANCE_INTERVAL = 3
class JOBOBJECT_SECURITY_LIMIT_INFORMATION(Structure):
    SecurityLimitFlags: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_SECURITY
    JobToken: win32more.Windows.Win32.Foundation.HANDLE
    SidsToDisable: POINTER(win32more.Windows.Win32.Security.TOKEN_GROUPS)
    PrivilegesToDelete: POINTER(win32more.Windows.Win32.Security.TOKEN_PRIVILEGES)
    RestrictedSids: POINTER(win32more.Windows.Win32.Security.TOKEN_GROUPS)
JOB_OBJECT_CPU_RATE_CONTROL = UInt32
JOB_OBJECT_CPU_RATE_CONTROL_ENABLE: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_CPU_RATE_CONTROL = 1
JOB_OBJECT_CPU_RATE_CONTROL_WEIGHT_BASED: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_CPU_RATE_CONTROL = 2
JOB_OBJECT_CPU_RATE_CONTROL_HARD_CAP: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_CPU_RATE_CONTROL = 4
JOB_OBJECT_CPU_RATE_CONTROL_NOTIFY: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_CPU_RATE_CONTROL = 8
JOB_OBJECT_CPU_RATE_CONTROL_MIN_MAX_RATE: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_CPU_RATE_CONTROL = 16
JOB_OBJECT_CPU_RATE_CONTROL_VALID_FLAGS: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_CPU_RATE_CONTROL = 31
JOB_OBJECT_IO_RATE_CONTROL_FLAGS = Int32
JOB_OBJECT_IO_RATE_CONTROL_ENABLE: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_IO_RATE_CONTROL_FLAGS = 1
JOB_OBJECT_IO_RATE_CONTROL_STANDALONE_VOLUME: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_IO_RATE_CONTROL_FLAGS = 2
JOB_OBJECT_IO_RATE_CONTROL_FORCE_UNIT_ACCESS_ALL: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_IO_RATE_CONTROL_FLAGS = 4
JOB_OBJECT_IO_RATE_CONTROL_FORCE_UNIT_ACCESS_ON_SOFT_CAP: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_IO_RATE_CONTROL_FLAGS = 8
JOB_OBJECT_IO_RATE_CONTROL_VALID_FLAGS: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_IO_RATE_CONTROL_FLAGS = 15
JOB_OBJECT_LIMIT = UInt32
JOB_OBJECT_LIMIT_WORKINGSET: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 1
JOB_OBJECT_LIMIT_PROCESS_TIME: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 2
JOB_OBJECT_LIMIT_JOB_TIME: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 4
JOB_OBJECT_LIMIT_ACTIVE_PROCESS: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 8
JOB_OBJECT_LIMIT_AFFINITY: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 16
JOB_OBJECT_LIMIT_PRIORITY_CLASS: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 32
JOB_OBJECT_LIMIT_PRESERVE_JOB_TIME: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 64
JOB_OBJECT_LIMIT_SCHEDULING_CLASS: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 128
JOB_OBJECT_LIMIT_PROCESS_MEMORY: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 256
JOB_OBJECT_LIMIT_JOB_MEMORY: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 512
JOB_OBJECT_LIMIT_JOB_MEMORY_HIGH: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 512
JOB_OBJECT_LIMIT_DIE_ON_UNHANDLED_EXCEPTION: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 1024
JOB_OBJECT_LIMIT_BREAKAWAY_OK: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 2048
JOB_OBJECT_LIMIT_SILENT_BREAKAWAY_OK: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 4096
JOB_OBJECT_LIMIT_KILL_ON_JOB_CLOSE: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 8192
JOB_OBJECT_LIMIT_SUBSET_AFFINITY: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 16384
JOB_OBJECT_LIMIT_JOB_MEMORY_LOW: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 32768
JOB_OBJECT_LIMIT_JOB_READ_BYTES: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 65536
JOB_OBJECT_LIMIT_JOB_WRITE_BYTES: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 131072
JOB_OBJECT_LIMIT_RATE_CONTROL: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 262144
JOB_OBJECT_LIMIT_CPU_RATE_CONTROL: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 262144
JOB_OBJECT_LIMIT_IO_RATE_CONTROL: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 524288
JOB_OBJECT_LIMIT_NET_RATE_CONTROL: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 1048576
JOB_OBJECT_LIMIT_VALID_FLAGS: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 524287
JOB_OBJECT_BASIC_LIMIT_VALID_FLAGS: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 255
JOB_OBJECT_EXTENDED_LIMIT_VALID_FLAGS: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 32767
JOB_OBJECT_NOTIFICATION_LIMIT_VALID_FLAGS: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_LIMIT = 2064900
JOB_OBJECT_NET_RATE_CONTROL_FLAGS = Int32
JOB_OBJECT_NET_RATE_CONTROL_ENABLE: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_NET_RATE_CONTROL_FLAGS = 1
JOB_OBJECT_NET_RATE_CONTROL_MAX_BANDWIDTH: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_NET_RATE_CONTROL_FLAGS = 2
JOB_OBJECT_NET_RATE_CONTROL_DSCP_TAG: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_NET_RATE_CONTROL_FLAGS = 4
JOB_OBJECT_NET_RATE_CONTROL_VALID_FLAGS: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_NET_RATE_CONTROL_FLAGS = 7
JOB_OBJECT_SECURITY = UInt32
JOB_OBJECT_SECURITY_NO_ADMIN: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_SECURITY = 1
JOB_OBJECT_SECURITY_RESTRICTED_TOKEN: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_SECURITY = 2
JOB_OBJECT_SECURITY_ONLY_TOKEN: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_SECURITY = 4
JOB_OBJECT_SECURITY_FILTER_TOKENS: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_SECURITY = 8
JOB_OBJECT_SECURITY_VALID_FLAGS: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_SECURITY = 15
JOB_OBJECT_TERMINATE_AT_END_ACTION = UInt32
JOB_OBJECT_TERMINATE_AT_END_OF_JOB: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_TERMINATE_AT_END_ACTION = 0
JOB_OBJECT_POST_AT_END_OF_JOB: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_TERMINATE_AT_END_ACTION = 1
JOB_OBJECT_UILIMIT = UInt32
JOB_OBJECT_UILIMIT_NONE: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_UILIMIT = 0
JOB_OBJECT_UILIMIT_HANDLES: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_UILIMIT = 1
JOB_OBJECT_UILIMIT_READCLIPBOARD: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_UILIMIT = 2
JOB_OBJECT_UILIMIT_WRITECLIPBOARD: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_UILIMIT = 4
JOB_OBJECT_UILIMIT_SYSTEMPARAMETERS: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_UILIMIT = 8
JOB_OBJECT_UILIMIT_DISPLAYSETTINGS: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_UILIMIT = 16
JOB_OBJECT_UILIMIT_GLOBALATOMS: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_UILIMIT = 32
JOB_OBJECT_UILIMIT_DESKTOP: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_UILIMIT = 64
JOB_OBJECT_UILIMIT_EXITWINDOWS: win32more.Windows.Win32.System.JobObjects.JOB_OBJECT_UILIMIT = 128
class JOB_SET_ARRAY(Structure):
    JobHandle: win32more.Windows.Win32.Foundation.HANDLE
    MemberLevel: UInt32
    Flags: UInt32


make_ready(__name__)



from __future__ import annotations
from win32more import ARCH, Annotated, Boolean, Byte, Bytes, Char, ComPtr, ConstantLazyLoader, Double, Enum, FAILED, Guid, Int16, Int32, Int64, IntPtr, POINTER, SByte, SUCCEEDED, Single, String, Structure, UInt16, UInt32, UInt64, UIntPtr, UnicodeAlias, Union, Void, VoidPtr, cfunctype, cfunctype_pointer, commethod, make_ready, winfunctype, winfunctype_pointer
from win32more._winrt import AwaitableProtocol, ContextManagerProtocol, FillArray, Generic, IterableProtocol, K, MappingProtocol, MulticastDelegate, PassArray, ReceiveArray, SequenceProtocol, T, TProgress, TResult, TSender, Tuple, V, WinRT_String, event, winrt_activatemethod, winrt_classmethod, winrt_commethod, winrt_factorymethod, winrt_mixinmethod, winrt_overload
import win32more.Windows.Devices.Geolocation
import win32more.Windows.Foundation
import win32more.Windows.Foundation.Collections
import win32more.Windows.Storage
import win32more.Windows.Storage.FileProperties
import win32more.Windows.Storage.Streams
import win32more.Windows.Win32.System.WinRT
class BasicProperties(ComPtr):
    extends: win32more.Windows.Win32.System.WinRT.IInspectable
    default_interface: win32more.Windows.Storage.FileProperties.IBasicProperties
    _classid_ = 'Windows.Storage.FileProperties.BasicProperties'
    @winrt_mixinmethod
    def get_Size(self: win32more.Windows.Storage.FileProperties.IBasicProperties) -> UInt64: ...
    @winrt_mixinmethod
    def get_DateModified(self: win32more.Windows.Storage.FileProperties.IBasicProperties) -> win32more.Windows.Foundation.DateTime: ...
    @winrt_mixinmethod
    def get_ItemDate(self: win32more.Windows.Storage.FileProperties.IBasicProperties) -> win32more.Windows.Foundation.DateTime: ...
    @winrt_mixinmethod
    def RetrievePropertiesAsync(self: win32more.Windows.Storage.FileProperties.IStorageItemExtraProperties, propertiesToRetrieve: win32more.Windows.Foundation.Collections.IIterable[WinRT_String]) -> win32more.Windows.Foundation.IAsyncOperation[win32more.Windows.Foundation.Collections.IMap[WinRT_String, win32more.Windows.Win32.System.WinRT.IInspectable]]: ...
    @winrt_mixinmethod
    def SavePropertiesAsync(self: win32more.Windows.Storage.FileProperties.IStorageItemExtraProperties, propertiesToSave: win32more.Windows.Foundation.Collections.IIterable[win32more.Windows.Foundation.Collections.IKeyValuePair[WinRT_String, win32more.Windows.Win32.System.WinRT.IInspectable]]) -> win32more.Windows.Foundation.IAsyncAction: ...
    @winrt_mixinmethod
    def SavePropertiesAsyncOverloadDefault(self: win32more.Windows.Storage.FileProperties.IStorageItemExtraProperties) -> win32more.Windows.Foundation.IAsyncAction: ...
    DateModified = property(get_DateModified, None)
    ItemDate = property(get_ItemDate, None)
    Size = property(get_Size, None)
class DocumentProperties(ComPtr):
    extends: win32more.Windows.Win32.System.WinRT.IInspectable
    default_interface: win32more.Windows.Storage.FileProperties.IDocumentProperties
    _classid_ = 'Windows.Storage.FileProperties.DocumentProperties'
    @winrt_mixinmethod
    def get_Author(self: win32more.Windows.Storage.FileProperties.IDocumentProperties) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_mixinmethod
    def get_Title(self: win32more.Windows.Storage.FileProperties.IDocumentProperties) -> WinRT_String: ...
    @winrt_mixinmethod
    def put_Title(self: win32more.Windows.Storage.FileProperties.IDocumentProperties, value: WinRT_String) -> Void: ...
    @winrt_mixinmethod
    def get_Keywords(self: win32more.Windows.Storage.FileProperties.IDocumentProperties) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_mixinmethod
    def get_Comment(self: win32more.Windows.Storage.FileProperties.IDocumentProperties) -> WinRT_String: ...
    @winrt_mixinmethod
    def put_Comment(self: win32more.Windows.Storage.FileProperties.IDocumentProperties, value: WinRT_String) -> Void: ...
    @winrt_mixinmethod
    def RetrievePropertiesAsync(self: win32more.Windows.Storage.FileProperties.IStorageItemExtraProperties, propertiesToRetrieve: win32more.Windows.Foundation.Collections.IIterable[WinRT_String]) -> win32more.Windows.Foundation.IAsyncOperation[win32more.Windows.Foundation.Collections.IMap[WinRT_String, win32more.Windows.Win32.System.WinRT.IInspectable]]: ...
    @winrt_mixinmethod
    def SavePropertiesAsync(self: win32more.Windows.Storage.FileProperties.IStorageItemExtraProperties, propertiesToSave: win32more.Windows.Foundation.Collections.IIterable[win32more.Windows.Foundation.Collections.IKeyValuePair[WinRT_String, win32more.Windows.Win32.System.WinRT.IInspectable]]) -> win32more.Windows.Foundation.IAsyncAction: ...
    @winrt_mixinmethod
    def SavePropertiesAsyncOverloadDefault(self: win32more.Windows.Storage.FileProperties.IStorageItemExtraProperties) -> win32more.Windows.Foundation.IAsyncAction: ...
    Author = property(get_Author, None)
    Comment = property(get_Comment, put_Comment)
    Keywords = property(get_Keywords, None)
    Title = property(get_Title, put_Title)
class GeotagHelper(ComPtr):
    extends: win32more.Windows.Win32.System.WinRT.IInspectable
    _classid_ = 'Windows.Storage.FileProperties.GeotagHelper'
    @winrt_classmethod
    def GetGeotagAsync(cls: win32more.Windows.Storage.FileProperties.IGeotagHelperStatics, file: win32more.Windows.Storage.IStorageFile) -> win32more.Windows.Foundation.IAsyncOperation[win32more.Windows.Devices.Geolocation.Geopoint]: ...
    @winrt_classmethod
    def SetGeotagFromGeolocatorAsync(cls: win32more.Windows.Storage.FileProperties.IGeotagHelperStatics, file: win32more.Windows.Storage.IStorageFile, geolocator: win32more.Windows.Devices.Geolocation.Geolocator) -> win32more.Windows.Foundation.IAsyncAction: ...
    @winrt_classmethod
    def SetGeotagAsync(cls: win32more.Windows.Storage.FileProperties.IGeotagHelperStatics, file: win32more.Windows.Storage.IStorageFile, geopoint: win32more.Windows.Devices.Geolocation.Geopoint) -> win32more.Windows.Foundation.IAsyncAction: ...
class IBasicProperties(ComPtr):
    extends: win32more.Windows.Win32.System.WinRT.IInspectable
    _classid_ = 'Windows.Storage.FileProperties.IBasicProperties'
    _iid_ = Guid('{d05d55db-785e-4a66-be02-9beec58aea81}')
    @winrt_commethod(6)
    def get_Size(self) -> UInt64: ...
    @winrt_commethod(7)
    def get_DateModified(self) -> win32more.Windows.Foundation.DateTime: ...
    @winrt_commethod(8)
    def get_ItemDate(self) -> win32more.Windows.Foundation.DateTime: ...
    DateModified = property(get_DateModified, None)
    ItemDate = property(get_ItemDate, None)
    Size = property(get_Size, None)
class IDocumentProperties(ComPtr):
    extends: win32more.Windows.Win32.System.WinRT.IInspectable
    _classid_ = 'Windows.Storage.FileProperties.IDocumentProperties'
    _iid_ = Guid('{7eab19bc-1821-4923-b4a9-0aea404d0070}')
    @winrt_commethod(6)
    def get_Author(self) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_commethod(7)
    def get_Title(self) -> WinRT_String: ...
    @winrt_commethod(8)
    def put_Title(self, value: WinRT_String) -> Void: ...
    @winrt_commethod(9)
    def get_Keywords(self) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_commethod(10)
    def get_Comment(self) -> WinRT_String: ...
    @winrt_commethod(11)
    def put_Comment(self, value: WinRT_String) -> Void: ...
    Author = property(get_Author, None)
    Comment = property(get_Comment, put_Comment)
    Keywords = property(get_Keywords, None)
    Title = property(get_Title, put_Title)
class IGeotagHelperStatics(ComPtr):
    extends: win32more.Windows.Win32.System.WinRT.IInspectable
    _classid_ = 'Windows.Storage.FileProperties.IGeotagHelperStatics'
    _iid_ = Guid('{41493244-2524-4655-86a6-ed16f5fc716b}')
    @winrt_commethod(6)
    def GetGeotagAsync(self, file: win32more.Windows.Storage.IStorageFile) -> win32more.Windows.Foundation.IAsyncOperation[win32more.Windows.Devices.Geolocation.Geopoint]: ...
    @winrt_commethod(7)
    def SetGeotagFromGeolocatorAsync(self, file: win32more.Windows.Storage.IStorageFile, geolocator: win32more.Windows.Devices.Geolocation.Geolocator) -> win32more.Windows.Foundation.IAsyncAction: ...
    @winrt_commethod(8)
    def SetGeotagAsync(self, file: win32more.Windows.Storage.IStorageFile, geopoint: win32more.Windows.Devices.Geolocation.Geopoint) -> win32more.Windows.Foundation.IAsyncAction: ...
class IImageProperties(ComPtr):
    extends: win32more.Windows.Win32.System.WinRT.IInspectable
    _classid_ = 'Windows.Storage.FileProperties.IImageProperties'
    _iid_ = Guid('{523c9424-fcff-4275-afee-ecdb9ab47973}')
    @winrt_commethod(6)
    def get_Rating(self) -> UInt32: ...
    @winrt_commethod(7)
    def put_Rating(self, value: UInt32) -> Void: ...
    @winrt_commethod(8)
    def get_Keywords(self) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_commethod(9)
    def get_DateTaken(self) -> win32more.Windows.Foundation.DateTime: ...
    @winrt_commethod(10)
    def put_DateTaken(self, value: win32more.Windows.Foundation.DateTime) -> Void: ...
    @winrt_commethod(11)
    def get_Width(self) -> UInt32: ...
    @winrt_commethod(12)
    def get_Height(self) -> UInt32: ...
    @winrt_commethod(13)
    def get_Title(self) -> WinRT_String: ...
    @winrt_commethod(14)
    def put_Title(self, value: WinRT_String) -> Void: ...
    @winrt_commethod(15)
    def get_Latitude(self) -> win32more.Windows.Foundation.IReference[Double]: ...
    @winrt_commethod(16)
    def get_Longitude(self) -> win32more.Windows.Foundation.IReference[Double]: ...
    @winrt_commethod(17)
    def get_CameraManufacturer(self) -> WinRT_String: ...
    @winrt_commethod(18)
    def put_CameraManufacturer(self, value: WinRT_String) -> Void: ...
    @winrt_commethod(19)
    def get_CameraModel(self) -> WinRT_String: ...
    @winrt_commethod(20)
    def put_CameraModel(self, value: WinRT_String) -> Void: ...
    @winrt_commethod(21)
    def get_Orientation(self) -> win32more.Windows.Storage.FileProperties.PhotoOrientation: ...
    @winrt_commethod(22)
    def get_PeopleNames(self) -> win32more.Windows.Foundation.Collections.IVectorView[WinRT_String]: ...
    CameraManufacturer = property(get_CameraManufacturer, put_CameraManufacturer)
    CameraModel = property(get_CameraModel, put_CameraModel)
    DateTaken = property(get_DateTaken, put_DateTaken)
    Height = property(get_Height, None)
    Keywords = property(get_Keywords, None)
    Latitude = property(get_Latitude, None)
    Longitude = property(get_Longitude, None)
    Orientation = property(get_Orientation, None)
    PeopleNames = property(get_PeopleNames, None)
    Rating = property(get_Rating, put_Rating)
    Title = property(get_Title, put_Title)
    Width = property(get_Width, None)
class IMusicProperties(ComPtr):
    extends: win32more.Windows.Win32.System.WinRT.IInspectable
    _classid_ = 'Windows.Storage.FileProperties.IMusicProperties'
    _iid_ = Guid('{bc8aab62-66ec-419a-bc5d-ca65a4cb46da}')
    @winrt_commethod(6)
    def get_Album(self) -> WinRT_String: ...
    @winrt_commethod(7)
    def put_Album(self, value: WinRT_String) -> Void: ...
    @winrt_commethod(8)
    def get_Artist(self) -> WinRT_String: ...
    @winrt_commethod(9)
    def put_Artist(self, value: WinRT_String) -> Void: ...
    @winrt_commethod(10)
    def get_Genre(self) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_commethod(11)
    def get_TrackNumber(self) -> UInt32: ...
    @winrt_commethod(12)
    def put_TrackNumber(self, value: UInt32) -> Void: ...
    @winrt_commethod(13)
    def get_Title(self) -> WinRT_String: ...
    @winrt_commethod(14)
    def put_Title(self, value: WinRT_String) -> Void: ...
    @winrt_commethod(15)
    def get_Rating(self) -> UInt32: ...
    @winrt_commethod(16)
    def put_Rating(self, value: UInt32) -> Void: ...
    @winrt_commethod(17)
    def get_Duration(self) -> win32more.Windows.Foundation.TimeSpan: ...
    @winrt_commethod(18)
    def get_Bitrate(self) -> UInt32: ...
    @winrt_commethod(19)
    def get_AlbumArtist(self) -> WinRT_String: ...
    @winrt_commethod(20)
    def put_AlbumArtist(self, value: WinRT_String) -> Void: ...
    @winrt_commethod(21)
    def get_Composers(self) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_commethod(22)
    def get_Conductors(self) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_commethod(23)
    def get_Subtitle(self) -> WinRT_String: ...
    @winrt_commethod(24)
    def put_Subtitle(self, value: WinRT_String) -> Void: ...
    @winrt_commethod(25)
    def get_Producers(self) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_commethod(26)
    def get_Publisher(self) -> WinRT_String: ...
    @winrt_commethod(27)
    def put_Publisher(self, value: WinRT_String) -> Void: ...
    @winrt_commethod(28)
    def get_Writers(self) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_commethod(29)
    def get_Year(self) -> UInt32: ...
    @winrt_commethod(30)
    def put_Year(self, value: UInt32) -> Void: ...
    Album = property(get_Album, put_Album)
    AlbumArtist = property(get_AlbumArtist, put_AlbumArtist)
    Artist = property(get_Artist, put_Artist)
    Bitrate = property(get_Bitrate, None)
    Composers = property(get_Composers, None)
    Conductors = property(get_Conductors, None)
    Duration = property(get_Duration, None)
    Genre = property(get_Genre, None)
    Producers = property(get_Producers, None)
    Publisher = property(get_Publisher, put_Publisher)
    Rating = property(get_Rating, put_Rating)
    Subtitle = property(get_Subtitle, put_Subtitle)
    Title = property(get_Title, put_Title)
    TrackNumber = property(get_TrackNumber, put_TrackNumber)
    Writers = property(get_Writers, None)
    Year = property(get_Year, put_Year)
class IStorageItemContentProperties(ComPtr):
    extends: win32more.Windows.Win32.System.WinRT.IInspectable
    _classid_ = 'Windows.Storage.FileProperties.IStorageItemContentProperties'
    _iid_ = Guid('{05294bad-bc38-48bf-85d7-770e0e2ae0ba}')
    @winrt_commethod(6)
    def GetMusicPropertiesAsync(self) -> win32more.Windows.Foundation.IAsyncOperation[win32more.Windows.Storage.FileProperties.MusicProperties]: ...
    @winrt_commethod(7)
    def GetVideoPropertiesAsync(self) -> win32more.Windows.Foundation.IAsyncOperation[win32more.Windows.Storage.FileProperties.VideoProperties]: ...
    @winrt_commethod(8)
    def GetImagePropertiesAsync(self) -> win32more.Windows.Foundation.IAsyncOperation[win32more.Windows.Storage.FileProperties.ImageProperties]: ...
    @winrt_commethod(9)
    def GetDocumentPropertiesAsync(self) -> win32more.Windows.Foundation.IAsyncOperation[win32more.Windows.Storage.FileProperties.DocumentProperties]: ...
class IStorageItemExtraProperties(ComPtr):
    extends: win32more.Windows.Win32.System.WinRT.IInspectable
    _classid_ = 'Windows.Storage.FileProperties.IStorageItemExtraProperties'
    _iid_ = Guid('{c54361b2-54cd-432b-bdbc-4b19c4b470d7}')
    @winrt_commethod(6)
    def RetrievePropertiesAsync(self, propertiesToRetrieve: win32more.Windows.Foundation.Collections.IIterable[WinRT_String]) -> win32more.Windows.Foundation.IAsyncOperation[win32more.Windows.Foundation.Collections.IMap[WinRT_String, win32more.Windows.Win32.System.WinRT.IInspectable]]: ...
    @winrt_commethod(7)
    def SavePropertiesAsync(self, propertiesToSave: win32more.Windows.Foundation.Collections.IIterable[win32more.Windows.Foundation.Collections.IKeyValuePair[WinRT_String, win32more.Windows.Win32.System.WinRT.IInspectable]]) -> win32more.Windows.Foundation.IAsyncAction: ...
    @winrt_commethod(8)
    def SavePropertiesAsyncOverloadDefault(self) -> win32more.Windows.Foundation.IAsyncAction: ...
class IThumbnailProperties(ComPtr):
    extends: win32more.Windows.Win32.System.WinRT.IInspectable
    _classid_ = 'Windows.Storage.FileProperties.IThumbnailProperties'
    _iid_ = Guid('{693dd42f-dbe7-49b5-b3b3-2893ac5d3423}')
    @winrt_commethod(6)
    def get_OriginalWidth(self) -> UInt32: ...
    @winrt_commethod(7)
    def get_OriginalHeight(self) -> UInt32: ...
    @winrt_commethod(8)
    def get_ReturnedSmallerCachedSize(self) -> Boolean: ...
    @winrt_commethod(9)
    def get_Type(self) -> win32more.Windows.Storage.FileProperties.ThumbnailType: ...
    OriginalHeight = property(get_OriginalHeight, None)
    OriginalWidth = property(get_OriginalWidth, None)
    ReturnedSmallerCachedSize = property(get_ReturnedSmallerCachedSize, None)
    Type = property(get_Type, None)
class IVideoProperties(ComPtr):
    extends: win32more.Windows.Win32.System.WinRT.IInspectable
    _classid_ = 'Windows.Storage.FileProperties.IVideoProperties'
    _iid_ = Guid('{719ae507-68de-4db8-97de-49998c059f2f}')
    @winrt_commethod(6)
    def get_Rating(self) -> UInt32: ...
    @winrt_commethod(7)
    def put_Rating(self, value: UInt32) -> Void: ...
    @winrt_commethod(8)
    def get_Keywords(self) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_commethod(9)
    def get_Width(self) -> UInt32: ...
    @winrt_commethod(10)
    def get_Height(self) -> UInt32: ...
    @winrt_commethod(11)
    def get_Duration(self) -> win32more.Windows.Foundation.TimeSpan: ...
    @winrt_commethod(12)
    def get_Latitude(self) -> win32more.Windows.Foundation.IReference[Double]: ...
    @winrt_commethod(13)
    def get_Longitude(self) -> win32more.Windows.Foundation.IReference[Double]: ...
    @winrt_commethod(14)
    def get_Title(self) -> WinRT_String: ...
    @winrt_commethod(15)
    def put_Title(self, value: WinRT_String) -> Void: ...
    @winrt_commethod(16)
    def get_Subtitle(self) -> WinRT_String: ...
    @winrt_commethod(17)
    def put_Subtitle(self, value: WinRT_String) -> Void: ...
    @winrt_commethod(18)
    def get_Producers(self) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_commethod(19)
    def get_Publisher(self) -> WinRT_String: ...
    @winrt_commethod(20)
    def put_Publisher(self, value: WinRT_String) -> Void: ...
    @winrt_commethod(21)
    def get_Writers(self) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_commethod(22)
    def get_Year(self) -> UInt32: ...
    @winrt_commethod(23)
    def put_Year(self, value: UInt32) -> Void: ...
    @winrt_commethod(24)
    def get_Bitrate(self) -> UInt32: ...
    @winrt_commethod(25)
    def get_Directors(self) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_commethod(26)
    def get_Orientation(self) -> win32more.Windows.Storage.FileProperties.VideoOrientation: ...
    Bitrate = property(get_Bitrate, None)
    Directors = property(get_Directors, None)
    Duration = property(get_Duration, None)
    Height = property(get_Height, None)
    Keywords = property(get_Keywords, None)
    Latitude = property(get_Latitude, None)
    Longitude = property(get_Longitude, None)
    Orientation = property(get_Orientation, None)
    Producers = property(get_Producers, None)
    Publisher = property(get_Publisher, put_Publisher)
    Rating = property(get_Rating, put_Rating)
    Subtitle = property(get_Subtitle, put_Subtitle)
    Title = property(get_Title, put_Title)
    Width = property(get_Width, None)
    Writers = property(get_Writers, None)
    Year = property(get_Year, put_Year)
class ImageProperties(ComPtr):
    extends: win32more.Windows.Win32.System.WinRT.IInspectable
    default_interface: win32more.Windows.Storage.FileProperties.IImageProperties
    _classid_ = 'Windows.Storage.FileProperties.ImageProperties'
    @winrt_mixinmethod
    def get_Rating(self: win32more.Windows.Storage.FileProperties.IImageProperties) -> UInt32: ...
    @winrt_mixinmethod
    def put_Rating(self: win32more.Windows.Storage.FileProperties.IImageProperties, value: UInt32) -> Void: ...
    @winrt_mixinmethod
    def get_Keywords(self: win32more.Windows.Storage.FileProperties.IImageProperties) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_mixinmethod
    def get_DateTaken(self: win32more.Windows.Storage.FileProperties.IImageProperties) -> win32more.Windows.Foundation.DateTime: ...
    @winrt_mixinmethod
    def put_DateTaken(self: win32more.Windows.Storage.FileProperties.IImageProperties, value: win32more.Windows.Foundation.DateTime) -> Void: ...
    @winrt_mixinmethod
    def get_Width(self: win32more.Windows.Storage.FileProperties.IImageProperties) -> UInt32: ...
    @winrt_mixinmethod
    def get_Height(self: win32more.Windows.Storage.FileProperties.IImageProperties) -> UInt32: ...
    @winrt_mixinmethod
    def get_Title(self: win32more.Windows.Storage.FileProperties.IImageProperties) -> WinRT_String: ...
    @winrt_mixinmethod
    def put_Title(self: win32more.Windows.Storage.FileProperties.IImageProperties, value: WinRT_String) -> Void: ...
    @winrt_mixinmethod
    def get_Latitude(self: win32more.Windows.Storage.FileProperties.IImageProperties) -> win32more.Windows.Foundation.IReference[Double]: ...
    @winrt_mixinmethod
    def get_Longitude(self: win32more.Windows.Storage.FileProperties.IImageProperties) -> win32more.Windows.Foundation.IReference[Double]: ...
    @winrt_mixinmethod
    def get_CameraManufacturer(self: win32more.Windows.Storage.FileProperties.IImageProperties) -> WinRT_String: ...
    @winrt_mixinmethod
    def put_CameraManufacturer(self: win32more.Windows.Storage.FileProperties.IImageProperties, value: WinRT_String) -> Void: ...
    @winrt_mixinmethod
    def get_CameraModel(self: win32more.Windows.Storage.FileProperties.IImageProperties) -> WinRT_String: ...
    @winrt_mixinmethod
    def put_CameraModel(self: win32more.Windows.Storage.FileProperties.IImageProperties, value: WinRT_String) -> Void: ...
    @winrt_mixinmethod
    def get_Orientation(self: win32more.Windows.Storage.FileProperties.IImageProperties) -> win32more.Windows.Storage.FileProperties.PhotoOrientation: ...
    @winrt_mixinmethod
    def get_PeopleNames(self: win32more.Windows.Storage.FileProperties.IImageProperties) -> win32more.Windows.Foundation.Collections.IVectorView[WinRT_String]: ...
    @winrt_mixinmethod
    def RetrievePropertiesAsync(self: win32more.Windows.Storage.FileProperties.IStorageItemExtraProperties, propertiesToRetrieve: win32more.Windows.Foundation.Collections.IIterable[WinRT_String]) -> win32more.Windows.Foundation.IAsyncOperation[win32more.Windows.Foundation.Collections.IMap[WinRT_String, win32more.Windows.Win32.System.WinRT.IInspectable]]: ...
    @winrt_mixinmethod
    def SavePropertiesAsync(self: win32more.Windows.Storage.FileProperties.IStorageItemExtraProperties, propertiesToSave: win32more.Windows.Foundation.Collections.IIterable[win32more.Windows.Foundation.Collections.IKeyValuePair[WinRT_String, win32more.Windows.Win32.System.WinRT.IInspectable]]) -> win32more.Windows.Foundation.IAsyncAction: ...
    @winrt_mixinmethod
    def SavePropertiesAsyncOverloadDefault(self: win32more.Windows.Storage.FileProperties.IStorageItemExtraProperties) -> win32more.Windows.Foundation.IAsyncAction: ...
    CameraManufacturer = property(get_CameraManufacturer, put_CameraManufacturer)
    CameraModel = property(get_CameraModel, put_CameraModel)
    DateTaken = property(get_DateTaken, put_DateTaken)
    Height = property(get_Height, None)
    Keywords = property(get_Keywords, None)
    Latitude = property(get_Latitude, None)
    Longitude = property(get_Longitude, None)
    Orientation = property(get_Orientation, None)
    PeopleNames = property(get_PeopleNames, None)
    Rating = property(get_Rating, put_Rating)
    Title = property(get_Title, put_Title)
    Width = property(get_Width, None)
class MusicProperties(ComPtr):
    extends: win32more.Windows.Win32.System.WinRT.IInspectable
    default_interface: win32more.Windows.Storage.FileProperties.IMusicProperties
    _classid_ = 'Windows.Storage.FileProperties.MusicProperties'
    @winrt_mixinmethod
    def get_Album(self: win32more.Windows.Storage.FileProperties.IMusicProperties) -> WinRT_String: ...
    @winrt_mixinmethod
    def put_Album(self: win32more.Windows.Storage.FileProperties.IMusicProperties, value: WinRT_String) -> Void: ...
    @winrt_mixinmethod
    def get_Artist(self: win32more.Windows.Storage.FileProperties.IMusicProperties) -> WinRT_String: ...
    @winrt_mixinmethod
    def put_Artist(self: win32more.Windows.Storage.FileProperties.IMusicProperties, value: WinRT_String) -> Void: ...
    @winrt_mixinmethod
    def get_Genre(self: win32more.Windows.Storage.FileProperties.IMusicProperties) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_mixinmethod
    def get_TrackNumber(self: win32more.Windows.Storage.FileProperties.IMusicProperties) -> UInt32: ...
    @winrt_mixinmethod
    def put_TrackNumber(self: win32more.Windows.Storage.FileProperties.IMusicProperties, value: UInt32) -> Void: ...
    @winrt_mixinmethod
    def get_Title(self: win32more.Windows.Storage.FileProperties.IMusicProperties) -> WinRT_String: ...
    @winrt_mixinmethod
    def put_Title(self: win32more.Windows.Storage.FileProperties.IMusicProperties, value: WinRT_String) -> Void: ...
    @winrt_mixinmethod
    def get_Rating(self: win32more.Windows.Storage.FileProperties.IMusicProperties) -> UInt32: ...
    @winrt_mixinmethod
    def put_Rating(self: win32more.Windows.Storage.FileProperties.IMusicProperties, value: UInt32) -> Void: ...
    @winrt_mixinmethod
    def get_Duration(self: win32more.Windows.Storage.FileProperties.IMusicProperties) -> win32more.Windows.Foundation.TimeSpan: ...
    @winrt_mixinmethod
    def get_Bitrate(self: win32more.Windows.Storage.FileProperties.IMusicProperties) -> UInt32: ...
    @winrt_mixinmethod
    def get_AlbumArtist(self: win32more.Windows.Storage.FileProperties.IMusicProperties) -> WinRT_String: ...
    @winrt_mixinmethod
    def put_AlbumArtist(self: win32more.Windows.Storage.FileProperties.IMusicProperties, value: WinRT_String) -> Void: ...
    @winrt_mixinmethod
    def get_Composers(self: win32more.Windows.Storage.FileProperties.IMusicProperties) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_mixinmethod
    def get_Conductors(self: win32more.Windows.Storage.FileProperties.IMusicProperties) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_mixinmethod
    def get_Subtitle(self: win32more.Windows.Storage.FileProperties.IMusicProperties) -> WinRT_String: ...
    @winrt_mixinmethod
    def put_Subtitle(self: win32more.Windows.Storage.FileProperties.IMusicProperties, value: WinRT_String) -> Void: ...
    @winrt_mixinmethod
    def get_Producers(self: win32more.Windows.Storage.FileProperties.IMusicProperties) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_mixinmethod
    def get_Publisher(self: win32more.Windows.Storage.FileProperties.IMusicProperties) -> WinRT_String: ...
    @winrt_mixinmethod
    def put_Publisher(self: win32more.Windows.Storage.FileProperties.IMusicProperties, value: WinRT_String) -> Void: ...
    @winrt_mixinmethod
    def get_Writers(self: win32more.Windows.Storage.FileProperties.IMusicProperties) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_mixinmethod
    def get_Year(self: win32more.Windows.Storage.FileProperties.IMusicProperties) -> UInt32: ...
    @winrt_mixinmethod
    def put_Year(self: win32more.Windows.Storage.FileProperties.IMusicProperties, value: UInt32) -> Void: ...
    @winrt_mixinmethod
    def RetrievePropertiesAsync(self: win32more.Windows.Storage.FileProperties.IStorageItemExtraProperties, propertiesToRetrieve: win32more.Windows.Foundation.Collections.IIterable[WinRT_String]) -> win32more.Windows.Foundation.IAsyncOperation[win32more.Windows.Foundation.Collections.IMap[WinRT_String, win32more.Windows.Win32.System.WinRT.IInspectable]]: ...
    @winrt_mixinmethod
    def SavePropertiesAsync(self: win32more.Windows.Storage.FileProperties.IStorageItemExtraProperties, propertiesToSave: win32more.Windows.Foundation.Collections.IIterable[win32more.Windows.Foundation.Collections.IKeyValuePair[WinRT_String, win32more.Windows.Win32.System.WinRT.IInspectable]]) -> win32more.Windows.Foundation.IAsyncAction: ...
    @winrt_mixinmethod
    def SavePropertiesAsyncOverloadDefault(self: win32more.Windows.Storage.FileProperties.IStorageItemExtraProperties) -> win32more.Windows.Foundation.IAsyncAction: ...
    Album = property(get_Album, put_Album)
    AlbumArtist = property(get_AlbumArtist, put_AlbumArtist)
    Artist = property(get_Artist, put_Artist)
    Bitrate = property(get_Bitrate, None)
    Composers = property(get_Composers, None)
    Conductors = property(get_Conductors, None)
    Duration = property(get_Duration, None)
    Genre = property(get_Genre, None)
    Producers = property(get_Producers, None)
    Publisher = property(get_Publisher, put_Publisher)
    Rating = property(get_Rating, put_Rating)
    Subtitle = property(get_Subtitle, put_Subtitle)
    Title = property(get_Title, put_Title)
    TrackNumber = property(get_TrackNumber, put_TrackNumber)
    Writers = property(get_Writers, None)
    Year = property(get_Year, put_Year)
class PhotoOrientation(Enum, Int32):
    Unspecified = 0
    Normal = 1
    FlipHorizontal = 2
    Rotate180 = 3
    FlipVertical = 4
    Transpose = 5
    Rotate270 = 6
    Transverse = 7
    Rotate90 = 8
class PropertyPrefetchOptions(Enum, UInt32):
    None_ = 0
    MusicProperties = 1
    VideoProperties = 2
    ImageProperties = 4
    DocumentProperties = 8
    BasicProperties = 16
class StorageItemContentProperties(ComPtr):
    extends: win32more.Windows.Win32.System.WinRT.IInspectable
    default_interface: win32more.Windows.Storage.FileProperties.IStorageItemContentProperties
    _classid_ = 'Windows.Storage.FileProperties.StorageItemContentProperties'
    @winrt_mixinmethod
    def GetMusicPropertiesAsync(self: win32more.Windows.Storage.FileProperties.IStorageItemContentProperties) -> win32more.Windows.Foundation.IAsyncOperation[win32more.Windows.Storage.FileProperties.MusicProperties]: ...
    @winrt_mixinmethod
    def GetVideoPropertiesAsync(self: win32more.Windows.Storage.FileProperties.IStorageItemContentProperties) -> win32more.Windows.Foundation.IAsyncOperation[win32more.Windows.Storage.FileProperties.VideoProperties]: ...
    @winrt_mixinmethod
    def GetImagePropertiesAsync(self: win32more.Windows.Storage.FileProperties.IStorageItemContentProperties) -> win32more.Windows.Foundation.IAsyncOperation[win32more.Windows.Storage.FileProperties.ImageProperties]: ...
    @winrt_mixinmethod
    def GetDocumentPropertiesAsync(self: win32more.Windows.Storage.FileProperties.IStorageItemContentProperties) -> win32more.Windows.Foundation.IAsyncOperation[win32more.Windows.Storage.FileProperties.DocumentProperties]: ...
    @winrt_mixinmethod
    def RetrievePropertiesAsync(self: win32more.Windows.Storage.FileProperties.IStorageItemExtraProperties, propertiesToRetrieve: win32more.Windows.Foundation.Collections.IIterable[WinRT_String]) -> win32more.Windows.Foundation.IAsyncOperation[win32more.Windows.Foundation.Collections.IMap[WinRT_String, win32more.Windows.Win32.System.WinRT.IInspectable]]: ...
    @winrt_mixinmethod
    def SavePropertiesAsync(self: win32more.Windows.Storage.FileProperties.IStorageItemExtraProperties, propertiesToSave: win32more.Windows.Foundation.Collections.IIterable[win32more.Windows.Foundation.Collections.IKeyValuePair[WinRT_String, win32more.Windows.Win32.System.WinRT.IInspectable]]) -> win32more.Windows.Foundation.IAsyncAction: ...
    @winrt_mixinmethod
    def SavePropertiesAsyncOverloadDefault(self: win32more.Windows.Storage.FileProperties.IStorageItemExtraProperties) -> win32more.Windows.Foundation.IAsyncAction: ...
class StorageItemThumbnail(ComPtr):
    extends: win32more.Windows.Win32.System.WinRT.IInspectable
    implements: Tuple[ContextManagerProtocol]
    default_interface: win32more.Windows.Storage.Streams.IRandomAccessStreamWithContentType
    _classid_ = 'Windows.Storage.FileProperties.StorageItemThumbnail'
    @winrt_mixinmethod
    def get_Size(self: win32more.Windows.Storage.Streams.IRandomAccessStream) -> UInt64: ...
    @winrt_mixinmethod
    def put_Size(self: win32more.Windows.Storage.Streams.IRandomAccessStream, value: UInt64) -> Void: ...
    @winrt_mixinmethod
    def GetInputStreamAt(self: win32more.Windows.Storage.Streams.IRandomAccessStream, position: UInt64) -> win32more.Windows.Storage.Streams.IInputStream: ...
    @winrt_mixinmethod
    def GetOutputStreamAt(self: win32more.Windows.Storage.Streams.IRandomAccessStream, position: UInt64) -> win32more.Windows.Storage.Streams.IOutputStream: ...
    @winrt_mixinmethod
    def get_Position(self: win32more.Windows.Storage.Streams.IRandomAccessStream) -> UInt64: ...
    @winrt_mixinmethod
    def Seek(self: win32more.Windows.Storage.Streams.IRandomAccessStream, position: UInt64) -> Void: ...
    @winrt_mixinmethod
    def CloneStream(self: win32more.Windows.Storage.Streams.IRandomAccessStream) -> win32more.Windows.Storage.Streams.IRandomAccessStream: ...
    @winrt_mixinmethod
    def get_CanRead(self: win32more.Windows.Storage.Streams.IRandomAccessStream) -> Boolean: ...
    @winrt_mixinmethod
    def get_CanWrite(self: win32more.Windows.Storage.Streams.IRandomAccessStream) -> Boolean: ...
    @winrt_mixinmethod
    def Close(self: win32more.Windows.Foundation.IClosable) -> Void: ...
    @winrt_mixinmethod
    def ReadAsync(self: win32more.Windows.Storage.Streams.IInputStream, buffer: win32more.Windows.Storage.Streams.IBuffer, count: UInt32, options: win32more.Windows.Storage.Streams.InputStreamOptions) -> win32more.Windows.Foundation.IAsyncOperationWithProgress[win32more.Windows.Storage.Streams.IBuffer, UInt32]: ...
    @winrt_mixinmethod
    def WriteAsync(self: win32more.Windows.Storage.Streams.IOutputStream, buffer: win32more.Windows.Storage.Streams.IBuffer) -> win32more.Windows.Foundation.IAsyncOperationWithProgress[UInt32, UInt32]: ...
    @winrt_mixinmethod
    def FlushAsync(self: win32more.Windows.Storage.Streams.IOutputStream) -> win32more.Windows.Foundation.IAsyncOperation[Boolean]: ...
    @winrt_mixinmethod
    def get_ContentType(self: win32more.Windows.Storage.Streams.IContentTypeProvider) -> WinRT_String: ...
    @winrt_mixinmethod
    def get_OriginalWidth(self: win32more.Windows.Storage.FileProperties.IThumbnailProperties) -> UInt32: ...
    @winrt_mixinmethod
    def get_OriginalHeight(self: win32more.Windows.Storage.FileProperties.IThumbnailProperties) -> UInt32: ...
    @winrt_mixinmethod
    def get_ReturnedSmallerCachedSize(self: win32more.Windows.Storage.FileProperties.IThumbnailProperties) -> Boolean: ...
    @winrt_mixinmethod
    def get_Type(self: win32more.Windows.Storage.FileProperties.IThumbnailProperties) -> win32more.Windows.Storage.FileProperties.ThumbnailType: ...
    CanRead = property(get_CanRead, None)
    CanWrite = property(get_CanWrite, None)
    ContentType = property(get_ContentType, None)
    OriginalHeight = property(get_OriginalHeight, None)
    OriginalWidth = property(get_OriginalWidth, None)
    Position = property(get_Position, None)
    ReturnedSmallerCachedSize = property(get_ReturnedSmallerCachedSize, None)
    Size = property(get_Size, put_Size)
    Type = property(get_Type, None)
class ThumbnailMode(Enum, Int32):
    PicturesView = 0
    VideosView = 1
    MusicView = 2
    DocumentsView = 3
    ListView = 4
    SingleItem = 5
class ThumbnailOptions(Enum, UInt32):
    None_ = 0
    ReturnOnlyIfCached = 1
    ResizeThumbnail = 2
    UseCurrentScale = 4
class ThumbnailType(Enum, Int32):
    Image = 0
    Icon = 1
class VideoOrientation(Enum, Int32):
    Normal = 0
    Rotate90 = 90
    Rotate180 = 180
    Rotate270 = 270
class VideoProperties(ComPtr):
    extends: win32more.Windows.Win32.System.WinRT.IInspectable
    default_interface: win32more.Windows.Storage.FileProperties.IVideoProperties
    _classid_ = 'Windows.Storage.FileProperties.VideoProperties'
    @winrt_mixinmethod
    def get_Rating(self: win32more.Windows.Storage.FileProperties.IVideoProperties) -> UInt32: ...
    @winrt_mixinmethod
    def put_Rating(self: win32more.Windows.Storage.FileProperties.IVideoProperties, value: UInt32) -> Void: ...
    @winrt_mixinmethod
    def get_Keywords(self: win32more.Windows.Storage.FileProperties.IVideoProperties) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_mixinmethod
    def get_Width(self: win32more.Windows.Storage.FileProperties.IVideoProperties) -> UInt32: ...
    @winrt_mixinmethod
    def get_Height(self: win32more.Windows.Storage.FileProperties.IVideoProperties) -> UInt32: ...
    @winrt_mixinmethod
    def get_Duration(self: win32more.Windows.Storage.FileProperties.IVideoProperties) -> win32more.Windows.Foundation.TimeSpan: ...
    @winrt_mixinmethod
    def get_Latitude(self: win32more.Windows.Storage.FileProperties.IVideoProperties) -> win32more.Windows.Foundation.IReference[Double]: ...
    @winrt_mixinmethod
    def get_Longitude(self: win32more.Windows.Storage.FileProperties.IVideoProperties) -> win32more.Windows.Foundation.IReference[Double]: ...
    @winrt_mixinmethod
    def get_Title(self: win32more.Windows.Storage.FileProperties.IVideoProperties) -> WinRT_String: ...
    @winrt_mixinmethod
    def put_Title(self: win32more.Windows.Storage.FileProperties.IVideoProperties, value: WinRT_String) -> Void: ...
    @winrt_mixinmethod
    def get_Subtitle(self: win32more.Windows.Storage.FileProperties.IVideoProperties) -> WinRT_String: ...
    @winrt_mixinmethod
    def put_Subtitle(self: win32more.Windows.Storage.FileProperties.IVideoProperties, value: WinRT_String) -> Void: ...
    @winrt_mixinmethod
    def get_Producers(self: win32more.Windows.Storage.FileProperties.IVideoProperties) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_mixinmethod
    def get_Publisher(self: win32more.Windows.Storage.FileProperties.IVideoProperties) -> WinRT_String: ...
    @winrt_mixinmethod
    def put_Publisher(self: win32more.Windows.Storage.FileProperties.IVideoProperties, value: WinRT_String) -> Void: ...
    @winrt_mixinmethod
    def get_Writers(self: win32more.Windows.Storage.FileProperties.IVideoProperties) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_mixinmethod
    def get_Year(self: win32more.Windows.Storage.FileProperties.IVideoProperties) -> UInt32: ...
    @winrt_mixinmethod
    def put_Year(self: win32more.Windows.Storage.FileProperties.IVideoProperties, value: UInt32) -> Void: ...
    @winrt_mixinmethod
    def get_Bitrate(self: win32more.Windows.Storage.FileProperties.IVideoProperties) -> UInt32: ...
    @winrt_mixinmethod
    def get_Directors(self: win32more.Windows.Storage.FileProperties.IVideoProperties) -> win32more.Windows.Foundation.Collections.IVector[WinRT_String]: ...
    @winrt_mixinmethod
    def get_Orientation(self: win32more.Windows.Storage.FileProperties.IVideoProperties) -> win32more.Windows.Storage.FileProperties.VideoOrientation: ...
    @winrt_mixinmethod
    def RetrievePropertiesAsync(self: win32more.Windows.Storage.FileProperties.IStorageItemExtraProperties, propertiesToRetrieve: win32more.Windows.Foundation.Collections.IIterable[WinRT_String]) -> win32more.Windows.Foundation.IAsyncOperation[win32more.Windows.Foundation.Collections.IMap[WinRT_String, win32more.Windows.Win32.System.WinRT.IInspectable]]: ...
    @winrt_mixinmethod
    def SavePropertiesAsync(self: win32more.Windows.Storage.FileProperties.IStorageItemExtraProperties, propertiesToSave: win32more.Windows.Foundation.Collections.IIterable[win32more.Windows.Foundation.Collections.IKeyValuePair[WinRT_String, win32more.Windows.Win32.System.WinRT.IInspectable]]) -> win32more.Windows.Foundation.IAsyncAction: ...
    @winrt_mixinmethod
    def SavePropertiesAsyncOverloadDefault(self: win32more.Windows.Storage.FileProperties.IStorageItemExtraProperties) -> win32more.Windows.Foundation.IAsyncAction: ...
    Bitrate = property(get_Bitrate, None)
    Directors = property(get_Directors, None)
    Duration = property(get_Duration, None)
    Height = property(get_Height, None)
    Keywords = property(get_Keywords, None)
    Latitude = property(get_Latitude, None)
    Longitude = property(get_Longitude, None)
    Orientation = property(get_Orientation, None)
    Producers = property(get_Producers, None)
    Publisher = property(get_Publisher, put_Publisher)
    Rating = property(get_Rating, put_Rating)
    Subtitle = property(get_Subtitle, put_Subtitle)
    Title = property(get_Title, put_Title)
    Width = property(get_Width, None)
    Writers = property(get_Writers, None)
    Year = property(get_Year, put_Year)


make_ready(__name__)
